module de2lab2b(
  // Clock Input (50 MHz)
  input  CLOCK_50,
  //  Push Buttons
  input  [3:0]  KEY,
  //  DPDT Switches 
  input  [17:0]  SW,
  //  7-SEG Displays
  output  [6:0]  HEX0, HEX1, HEX2, HEX3, HEX4, HEX5, HEX6, HEX7,
  //  LEDs
  output  [8:0]  LEDG,  //  LED Green[8:0]
  output  [17:0]  LEDR,  //  LED Red[17:0]
  //  PS2 data and clock lines		
  inout	PS2_DAT,
  inout	PS2_CLK,
  //  GPIO Connections
  inout  [35:0]  GPIO_0, GPIO_1
);

//  set all inout ports to tri-state
assign  GPIO_0    =  36'hzzzzzzzzz;
assign  GPIO_1    =  36'hzzzzzzzzz;

wire RST;
assign RST = KEY[0];


// Connect dip switches to red LEDs
assign LEDR[17:0] = SW[17:0];

// turn off green LEDs
//assign LEDG = 0;

wire reset = 1'b0;
wire [7:0] scan_code;

reg [7:0] history[1:4];
wire read, scan_ready;

one_pulse oneshot1(read,scan_ready,CLOCK_50);


ps2_rcv rcv(
  .ps2_clk(PS2_CLK),
  .ps2_data(PS2_DAT),
  .clock50(CLOCK_50),
  .reset(reset),
  .read_ack(read),
  .write_ack(write_ack),
  .data_ready(scan_ready),
  .data(scan_code)
);


hex_7seg dsp0(history[1][3:0],HEX0);
hex_7seg dsp1(history[1][7:4],HEX1);

hex_7seg dsp2(history[2][3:0],HEX2);
hex_7seg dsp3(history[2][7:4],HEX3);

hex_7seg dsp4(history[3][3:0],HEX4);
hex_7seg dsp5(history[3][7:4],HEX5);

hex_7seg dsp6(history[4][3:0],HEX6);
hex_7seg dsp7(history[4][7:4],HEX7);



always @(posedge scan_ready, negedge RST)
if (!RST)
begin
	history[4] <= 4'h0;
	history[3] <= 4'h0;
	history[2] <= 4'h0;
	history[1] <= 4'h0;
end
else begin
	history[4] <= history[3];
	history[3] <= history[2];
	history[2] <= history[1];
	history[1] <= scan_code;
end
	

// blank remaining digits
/*
wire [6:0] blank = 7'b111_1111;
assign HEX2 = blank;
assign HEX3 = blank;
assign HEX4 = blank;
assign HEX5 = blank;
assign HEX6 = blank;
assign HEX7 = blank;
*/

reg start;
reg [7:0] command;
wire write_ack, wait_ack, timeout_error;
wire cmd1, cmd2;

assign LEDG[2] = start;
assign LEDG[1] = wait_ack;
assign LEDG[0] = timeout_error;


ps2_xmt xmt(
	.clock50(CLOCK_50), 
	.nRESET(RST),
	.command(command),
	.start(start),
	.wait_ack(wait_ack),
	.write_ack(write_ack),
	.timeout_error(timeout_error),
	.ps2_data(PS2_DAT),
	.ps2_clock(PS2_CLK) 
);



reg [3:0] ir;
reg [2:0] pc;
wire xmt_complete = (wait_ack && scan_ready);

one_pulse oneshot2(cmd1,~KEY[3],CLOCK_50);
one_pulse oneshot3(cmd2,~KEY[2],CLOCK_50);

wire [2:0] npc = pc + 1'b1;

wire [1:0] opcode = ir[3:2];

always @(posedge CLOCK_50, negedge RST)
	if (!RST) pc <= 3'b000;
	else case (opcode)
	2'h1: begin
			start <= 1'b1;
			command <= data;
			pc <= npc;
		  end
	2'h2: if (xmt_complete)
			begin
			  start <= 1'b0;
			  pc <= npc;
			end
	default:  
			begin
				start <= 1'b0;
				if (cmd1) pc <= 3'h1;
			  	else if (cmd2) pc <= 3'h4;
			end
	endcase
	
	assign LEDG[7:4] = pc;		 
		

always
	case (pc)
	3'h0: ir = 4'b0000; // halt
	3'h1: ir = 4'b0101; // write 1 (F2)
	3'h2: ir = 4'b1000; // wait (for acknowledgement signal)
	3'h3: ir = 4'b0000; // halt
	3'h4: ir = 4'b0110; // write 2 (ED)
	3'h5: ir = 4'b1000; // wait
	3'h6: ir = 4'b0111; // write 3 (switches)
	3'h7: ir = 4'b1000; // wait
	default: ir = 4'b0000; // halt
	endcase

reg [7:0] data;	
always 
	case (ir[1:0])
		2'b01: data <= 8'hF2;
		2'b10: data <= 8'hED;
		2'b11: data <= {5'h00,SW[17:15]};
		default: data <= 8'h00;
	endcase


endmodule