module ps2_xmt(input clock50, input nRESET,  
	input [7:0] command, input start,  
	inout ps2_data, inout ps2_clock,  output reg write_ack,
	output wait_ack, output timeout_error);
	
	
reg [7:0] filter;
reg clock25;
reg clock_posedge, filtered_clock;

// generate 25MHz clock from 50MHz clock
always @(posedge clock50)
	clock25 = ~clock25;

// Filter the PS2_clock signal
always @(posedge clock25)
	begin
	 filter <= {filter[6:0],ps2_clock};
	 if (filter==8'hff)
	 begin
	    clock_posedge <= (filtered_clock==1'b0);
	    filtered_clock <= 1'b1;
	 end
	 if (filter==8'h00)
	 begin
	    //clock_negedge = (filtered_clock==1'b1);
	    filtered_clock <= 1'b0;
	 end
	end

reg [19:0] counter;

reg [2:0] state;

parameter
	WAIT_INIT = 2500,     // 100 us (2500/25MHz)
	WAIT_TIMEOUT = 750000; // 30 ms

parameter
	IDLE = 3'h0,
	INIT = 3'h1,
	WAIT_FOR_CLOCK = 3'h2,
	TRANSMIT = 3'h3,
	WAIT_ACK = 3'h4,
	TIMEOUT = 3'h5;

always @(posedge clock25)
begin
	if (!nRESET || state==IDLE) counter = 20'h00000;
	else counter <= counter + 1'b1;
end


always @(posedge clock25)
	if (!nRESET) state <= IDLE;
	else case (state)
	IDLE:
			state <= (start? INIT: IDLE);
	INIT:
	   if (counter==WAIT_INIT) state <= WAIT_FOR_CLOCK;
	WAIT_FOR_CLOCK:
	   if (clock_posedge) state <= TRANSMIT;
	   else if (counter>=WAIT_TIMEOUT) state <= TIMEOUT;
	TRANSMIT:
	   if (write_ack) state <= WAIT_ACK;
	   else if (counter>=WAIT_TIMEOUT) state <= TIMEOUT;
	WAIT_ACK:
	   if (!start) state <= IDLE;
	TIMEOUT: state <= (!start? IDLE: TIMEOUT);
	default: state <= IDLE;
	endcase

wire parity = ~(^command);

reg [9:0] bits;
reg [3:0] bitcount;

always @ (negedge filtered_clock)
	if (state==TRANSMIT)
	begin
		if (bitcount<=4'b1001) 
		begin
			bitcount <= bitcount + 1'b1;
			bits <= {1'b1, bits[9:1]};
		end
		else write_ack <= 1'b1;
	end
	else
	begin
		bits <= {parity,command,1'b0};
		bitcount <= 4'h0;
		write_ack <= 1'b0;
	end

	assign ps2_clock = (state==INIT? 1'b0: 1'bz);
	assign ps2_data = (state==WAIT_FOR_CLOCK? 1'b0: (state==TRANSMIT? bits[0]: 1'bz));
	assign wait_ack = (state==WAIT_ACK);
	assign timeout_error = (state==TIMEOUT);
endmodule